var https = require('https');
var mqtt = require('mqtt');
var fs = require('fs');

const proficloud_api = "env.kaa.proficloud-production.io";
const ebjca_api = "/middleware/ejbca/api/";

function Connector(uuid) {
    var self = this;
    this.uuid = uuid;
    this.authBundle = "";
    this.client;
    this.connectCallback = function () { };
    this.connectionLostCallback = function () { };

    try {
        this.authBundle = JSON.parse(fs.readFileSync(__dirname + '/' + uuid + '_auth_bundle.json'));
    } catch (e) {

    }

    this.connect = function (connectCallback, connectionLostCallback) {
        self.getCertState();
        self.connectCallback = connectCallback;
        self.connectionLostCallback = connectionLostCallback;
    };

    this.disconnect = function () {
        this.client.disconnect();
    };

    this.sendTSD = function (tsd_data) {
        var tsd_topic = "kp1/proficloud-v1/dcx/" + this.authBundle.DeviceToken + "/json/" + Math.floor(Math.random() * Math.floor(1000));
        if (this.client != undefined) {
            console.log(JSON.stringify([tsd_data]));
            this.client.publish(tsd_topic, JSON.stringify([tsd_data]));
        }
    };

    this.sendMeta = function (meta_data) {
        var meta_topic = "kp1/proficloud-v1/epmx/" + this.authBundle.DeviceToken + "/update/keys/" + Math.floor(Math.random() * Math.floor(1000));
        if (this.client != undefined) {
            this.client.publish(meta_topic, JSON.stringify(meta_data));
        }
    }

    this.connectMQTT = function () {
        var mqtt_options = {
            key: this.authBundle.DeviceCertKey,
            cert: this.authBundle.DeviceCert,
            ca: this.authBundle.CACert,
            rejectUnauthorized: false // TODO wrong hostname in cert
        };
        if (this.authBundle.DeviceCertKey == undefined || this.authBundle.DeviceCert == undefined || this.authBundle.CACert == undefined){
            self.connectionLostCallback();
            return;
        }
        console.log(mqtt_options);
        this.client = mqtt.connect('mqtts://mqtt.proficloud-production.io:8883', mqtt_options);
        this.client.on("connect", function () {
            console.log("connected");
            self.connectCallback();
        });
        this.client.on("message", function (topic, msg) {
            console.log(topic + msg);
        });

        this.client.on("error", function (error) { 
            console.log("Can't connect" + error);
            self.connectionLostCallback();
        });
    };

    this.getCertState = function () {
        https.get("https://" + proficloud_api + ebjca_api + "certificate/status/" + this.uuid, (resp) => {
            let data = '';
            resp.on('data', (chunk) => {
                data += chunk;
            });

            resp.on('end', () => {
                console.log(JSON.parse(data));
                const dataObject = JSON.parse(data);
                if (dataObject.generated) {
                    this.connectMQTT();
                } else {
                    this.getNewCertificate();
                }
            });

        }).on("error", (err) => {
            console.log("Error: " + err.message);
        });
    };

    this.getNewCertificate = function () {
        var options = {
            hostname: proficloud_api,
            port: 443,
            path: ebjca_api + "certificate/device/" + uuid,
            method: 'PUT'
        };
        https.get(options, (resp) => {
            let data = '';
            resp.on('data', (chunk) => {
                data += chunk;
            });

            resp.on('end', () => {
                console.log(JSON.parse(data));
                this.authBundle = JSON.parse(data);
                fs.writeFile(__dirname + '/' + uuid + '_auth_bundle.json', JSON.stringify(this.authBundle), (err) => {
                    if (err) {
                        throw err;
                    }
                    console.log("JSON data is saved.");
                    this.connectMQTT();
                });

            });

        }).on("error", (err) => {
            console.log("Error: " + err.message);
        });
    };
}

module.exports.Connector = Connector;